// $Id: PerlInterpreterStub.H,v 1.1 2001/12/26 03:28:19 nolanw Exp $
// $Author: nolanw $
//
// This class is a singleton used to instantiate a perl interpreter so that
// the PerlStream library can use the instance to evaluate perl expressions.
// 
// The general usage is something like this:
// int main(int argc, char **argv, char **env) {
//   PerlInterpreterStub::instance()->loadInterpreter(env);
//   ...
// }
//
// Note that the class will automatically cleanup the interpreter when it is 
// destructed at the end of the program, as it is a static instance.
//
// Ideally, I could have the PerlStream library initialize this the first time
// round.

#ifndef __PERL_INTERPRETER_H
#define __PERL_INTERPRETER_H

#include <EXTERN.h>
#include <perl.h>

extern PerlInterpreter *my_perl;

class PerlInterpreterStub {
public:

  // Accessor function to obtain an instance of this singleton.
  static PerlInterpreterStub *instance();

  // Initialization function to setup the perl interpreter.  Needs only
  // to be called once, optionally with the environment pointer supplied in 
  // entry to "int main(int argc, char **argv, char **env)".
  // Destruction is automatic upon exit from the program.
  void loadInterpreter(char **env_ = NULL);
  
private:
  PerlInterpreterStub();
  ~PerlInterpreterStub();

  static PerlInterpreterStub *_instance;
};

#endif
