// $Id: testPerlCPP.C,v 1.1 2001/12/26 03:28:09 nolanw Exp $
// $Author: nolanw $
//
// Program to test the PerlStream classes.

#include <PerlStream.H>
#include <iostream.h>
#include <PerlInterpreterStub.H>

int main(int argc, char **argv, char **env) {
  PerlInterpreterStub::instance()->loadInterpreter();
  PerlStream ps;

  cout << "Test #1, executing \"10+5\":\n";

  ps << "10+5" << perl_execute;

  int retval = ps[0];

  cout << "Return value of 10+5 = " << retval << endl;

  cout << "Test #2, executing \"(2,5,{'a'=>'foo', 'b'=>'bar'})\":\n";

  ps.array() << "(2,5,{'a'=>'foo', 'b'=>'bar'})" << perl_execute;
  int two = ps[0];
  double five = ps[-2];
  string foo = ps[2]["a"];

  cout << two << "," << five << "," << foo << endl;
  
  cout << "Test #3, executing \"require Cwd;\":\n";

  ps.voidc() << "require Cwd;" << perl_execute;  // void context

  cout << "Test #4, attempting to refer to bad string of references:\n";

  // Bad string of references are tolerated
  int undefined = ps[0]["a"][-3]["bar"];         // undefined = 0

  cout << undefined << endl;

  cout << "Test #5, executing \"( [4, 9, 16], {'a'=>25, " 
	   << "'b'=>36, 'c'=>49} )\":\n";

  // iterate over an array and hash
  ps.array() << "( [4, 9, 16], {'a'=>25, 'b'=>36, 'c'=>49} )" <<
    perl_execute;
  PerlValue::array_iter ai;
  for (ai = ps[0].array_begin(); ai != ps[0].array_end(); ++ai) {
	if (ai->isInt())
	  cout << "Value = " << static_cast<int>(*ai) << endl;
  }
  PerlValue::hash_iter hi;
  for (hi = ps[1].hash_begin(); hi != ps[1].hash_end(); ++hi) {
	cout << "Key=" << hi->first << " Val=" << hi->second << endl;
  }
  
  // Cause an evaluation exception
  cout << "Test #6, attempting to cause evaluation exception:\n";

  ps.setExceptionFlags(PerlStream::eEvalFail);
  try {
	ps << "[ 10, 15" << perl_execute;       // Missing ']'
  }
  catch (exception &e) {
	cerr << "Error: " << e.what() << endl;  // e is PerlEvalError
  }

  // Define a function and then try to use it later
  cout << "Test #7, evaluating a sub declaration and then attempting to use it:\n";
  cout << "\"sub foo { my ($a) = @_; print \"$a\\n\"; }\"\n";

  ps << "sub foo { my ($a) = @_; print \"$a\\n\"; }" << perl_execute;
  ps << "foo(\"hello\");" << perl_execute;

  // Define another function that averages all the elements of an array,
  // and call it with a really huge array.
  cout << "Test #8, defining an averaging function and calling it with a 10,000 element array [1..10000]:\n";
  
  ps << "sub average { my ($a) = @_; my $tot = 0; my $n = scalar @$a; for(my $i = 0 ; $i < $n ; $i++) { $tot += $a->[$i]; } return $tot/$n; }" << perl_execute;

  string call;

  call.append("my $array = [");
  for(int i = 0 ; i < 10000 ; i++) {
	char buf[10];
	sprintf(buf, "%d", i);
	call.append(buf);
	if(i != 9999) {
	  call.append(",");
	}
  }
  call.append("]; print average($array); print \"\\n\";");

  try {
	ps << call << perl_execute;
  } catch (exception &e) {
	cerr << "Error: " << e.what() << endl;  // e is PerlEvalError
  }


  return 0;
}
